# 200415 Syllogist:Cell type deconvolution using RNA-seq data (FISHER TEST)
#
# README
# 
# BEFORE START
#
# 1) Required packages to run the script:
#
#          a.BiocGenerics (Bioconductor)
#          b.pheatmap (CRAN)
#          c.gplots (CRAN)
#          d.viridis (CRAN)
#          e.RColorBrewer (CRAN)
#
# 2) Required files in the working directory to run the script:
#       
#         a. The Reference Map ("200116_Reference map_Top 80 Specific genes for each cell type.csv") See also Supplementary Table 1
#         b. A list of gene names from which the script will perform random sampling (provided as "Affy_genes_unique3.csv")
#         c. The query data to be analyzed in .csv format (we provide 1 dataset "Example query 1_GSE59612_GBM cores vs normal brain tissues.csv". This example highlights differences between normal and glioblastoma tissue samples and is the same being used in Figure 2)
#         (To run your own query data, please provide a file with transcriptome data in .csv format with gene names (HUGO Symbols) as rows and samples as columns (see examples provided))
#
# 3) Required action: set working directory by updating the setwd() function accordingly at line 54 of this script
# 
# 4) script can be ideally run using the source button in RStudio
#
# EXECUTION
#
# 1) As soon as all required packages are installed and all required files are placed in the working directory, and the setwd() is updated accordingly, you can source the script via RStudio
# 2) A list of files contained in the working directory will be shown in the console and the source area of RStudio 
# 3) A first prompt asking "which data should I analyze ? --"  will be shown in the console. Enter the number of the file containing the query dataset as seen in the list  (e.g. 4.)
# 4) A second prompt will ask "Choose a threshold". This threshold is defined to distinguish between exprerssed genes and background noise in the transcriptome data and varies depending on the expression data used (in this case, use 0.1 for the example provided)
# 5) A third prompt will ask "How many Monte Carlo simulations?" for the number of simulations desired to run the null model. In the manuscript we used 1000 for all analyses. By using less simulations (e.g. 100), the script will run much faster but with less precision.
# 
# RESULTS
# 
# 1) Deconvolution data are stored in object "Fisher.t", which contains normalized odds ratios as proxy for the relative quantity of the respective cell type. This file contains normalized odds ratios
# IMPORTANT: the resulting odds ratios can be used to compare the same cell type between samples, but cannot be used to compare different cell types within one sample.
# 2) Raw odds ratios are also stored in object Fisher.t_unscaled
# 3) Heatmap is generated from the Fisher.t file to visualize data and shown in the plots


###
###
#
#  STEP 1:generates a file from query data containing list of genes with value > treshold defined by user
#
###
###

rm(list = ls())   # cleanup workspace

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
list.files(path = dirname(rstudioapi::getActiveDocumentContext()$path))


file <- readline("Which data should I analyze ? -- ")
a <- readline("Choose a threshold (e.g. suggested 0.1 for FPKM data) -- ")
sim <- readline("How many Monte Carlo simulations? (e.g. 100-1000) -- ")

D <- read.csv(list.files()[as.numeric(file)], header = TRUE)
r <- D[,1]
c <-colnames(D[,-1])
D <- D[,-1]

c <-colnames(D)

Dm <- as.matrix(D)
class(Dm) <- "numeric"
class(a) <- "numeric"
class(sim) <- "numeric"
rownames(Dm) <- r
colnames(Dm) <- c
ncol <- ncol(Dm)

Dm[which(Dm <= a)] <- NA #set expression values < = a as NA, e.g. for FPKM units in general 0.1 or higher, for RPKM in general 4 or higher, for TPM in general 20 or higher

# create vector of  counts of all elements in a column withouth NA 

Dcount <- colSums(!is.na(Dm))
n <- max(Dcount)

# matrix of names of genes present in a defined column

genes <- matrix(nrow=n)

for (i in 1:ncol) {
  names <- names(na.omit(Dm[,i]))
  length(names) <- n
  genes <- cbind(genes, names)
}

genes <- genes[,-1]
genes[is.na(genes)]<-""
colnames(genes) <- c
write.csv(genes, "gene names.csv")


###
###
#
#  STEP 2: for each query sample, generate a vector of genes commonly present in both sample and reference
#
###
###


specific <- read.csv("200116_Reference map_Top 80 Specific genes for each cell type.csv", header = T)
rnaseq <- read.csv("gene names.csv", header = T)
specific <- specific[,-1]

rnaseq <- rnaseq[,-1]
sn <- colnames(specific)
rn <- colnames(rnaseq)
ncol1 <- ncol(specific)
ncol2 <- ncol(rnaseq)

# generates 100 lists of 80 random genes to be used as random specificity enrichment samples
genes <- read.csv('Affy_genes_unique3.csv', header=T)
random.specific <- matrix(data=NA, ncol= sim, nrow=80)
for (i in 1:sim) {
  random.specific[,i] <- as.matrix(sample(genes[,2], 80, replace = F))
}


# number of genes enriched in rnaseq data and specificity 
df <- matrix(data=NA, ncol=ncol2, nrow=ncol1)
for (j in 1:ncol1) {
  for (i in 1:ncol2) {
    df[j,i] <- length(intersect(specific[,j],rnaseq[,i]))
  }}


#  random specificity
df.random <- matrix(data=NA, ncol=ncol2, nrow=sim)
for (j in 1:sim) {
  for (i in 1:ncol2) {
    df.random[j,i] <- length(intersect(random.specific[,j],rnaseq[,i])) 
  }}


###
###
#
#  STEP 3: Fisher's exact test. Generate a table with odds-ratios
#
###
###


Fisher <- matrix(data=NA, ncol=ncol2, nrow=ncol1)
for (j in 1:ncol1) {
  for (i in 1:ncol2) {
    f <- matrix(c(df[j,i],80, round(mean(df.random[,i])) , 80), ncol=2, byrow =T)
    Fisher[j,i] <- fisher.test(f, alternative = 'greater', conf.int=T)$estimate
  }}


###
###
#
#  STEP 4: create object with raw odds ratios (Fisher.t_unscaled),  normalized odds ratios (Fisher.t) and heatmap
#
###
###


# heatmap of normalized odds ratios. 

#normalization of the odds ratios: scale and center so that all values lie between 0 and 1
maxs <- apply(Fisher, 2, max)
mins <- apply(Fisher, 2, min)
Fisher.t_unscaled <-t(Fisher)

Fisher <- scale(Fisher, center = mins, scale = maxs - mins)

#transpose
Fisher.t <- t(Fisher)

library(BiocGenerics)
rownames(Fisher.t) <- rn
colnames(Fisher.t) <- sn

rownames(Fisher.t_unscaled) <- rn
colnames(Fisher.t_unscaled) <- sn


rownames(Fisher) <- sn
colnames(Fisher) <- rn

# heatmaps

library(pheatmap) 
library(gplots)
library(viridis)
library(RColorBrewer)

#define breaks so that coloring highlights normalized odds ratios >0.8
#colors <- c(seq(0,0.3,length=1),seq(0.301,0.45,length=90),seq(0.451,1, length=500))
colors <- c(seq(0,0.3,length=1),seq(0.301,0.65,length=90),seq(0.651,1, length=500))

#heatmap optionals: [-grep("endo", rownames(Fisher.t)),]

# color options
#colors "dodgerblue4", "white", "darkorange2"
# cividis, magma, inferno, viridis plasma
#"black', 'yellow', 'red'

pheatmap(Fisher.t, breaks = colors, color = colorpanel(600, "dodgerblue4", "white", "darkorange2"), legend = T, cluster_cols = F, cluster_rows = T, main = paste(list.files()[as.numeric(file)] ,". Treshold:", a), treeheight_row = 50, treeheight_col = 25, fontsize_row = 5, fontsize_col = 5)
pheatmap(Fisher.t, breaks = colors, color = cividis(600), legend = T, cluster_cols = F, cluster_rows = T, main = paste(list.files()[as.numeric(file)] ,". Treshold:", a), treeheight_row = 50, treeheight_col = 25, fontsize_row = 5, border_color = TRUE, fontsize_col = 5)

#write.csv(Fisher.t, "filename.csv")
#write.csv(Fisher.t_unscaled, "filename.csv")

system("say I am done!")

